import { Request, Response } from "express";
import bcrypt from "bcryptjs";
import jwt from "jsonwebtoken";
import prisma from "../database/dbclient";
import dotenv from "dotenv";

dotenv.config();

export const registerUser = async (req: Request, res: Response) => {
  const { email, password, fname, lname, middleName } = req.body;

  try {
    // Check if email already exists
    const existingUser = await prisma.users.findUnique({
      where: { email },
    });

    if (existingUser) {
      return res.status(400).json({ message: "Email already exists" });
    }

    // Hash the password
    const hashedPassword = await bcrypt.hash(password, 10);

    // Create the new user
    const newUser = await prisma.users.create({
      data: {
        email,
        password_hash: hashedPassword,
        first_name: fname,
        last_name: lname,
        middle_name: middleName,
      },
    });

    res.status(201).json({ message: "User registered successfully" });
  } catch (error) {
    res.status(500).json({ message: "Something Went Wrong", error });
  }
};

export const loginUser = async (req: Request, res: Response) => {
  const { email, password } = req.body;

  try {
    const   user = await prisma.users.findUnique({
      where: { email },
      include: {
        user_roles: {
          select: {
            roles: {
              select: { role_name: true , role_id : true },
            },
          
          },
        },
        access_permissions : {
          select : {
            permission: {
              select : {
                permission_name : true,
              }
            } ,
          }
        },
        // Company data is necessary to redirect employees and company admins
        usercompany: true,
      },
    });

     var permissionArrayMap = user?.access_permissions.map((item) => item.permission.permission_name )


    if(user?.user_roles?.roles.role_id == 1  || user?.user_roles?.roles.role_id == 3 ){
      // there users need all permissions 
     var  listOfMap  =  await prisma.permissions.findMany(
        {
          select : {
            permission_name : true 
          }
        }
      ) ;
      
       permissionArrayMap = listOfMap.map((item) => item.permission_name )

     
    }
  
   
   


    if (!user || !(await bcrypt.compare(password, user.password_hash))) {
      return res.status(401).json({ message: "Invalid email or password" });
    }

    const token = jwt.sign(
      { userId: user.user_id },
      process.env.JWT_SECRET as string,
      {
        expiresIn: process.env.JWT_EXPIRES_IN,
      }
    );

    user.password_hash = "";
      
    user.access_permissions = [] ; 



    res.json({ token: token, user: user , permissions : permissionArrayMap });
  } catch (error) {
    res.status(500).json({ message: "Error logging in", error });
  }
};


export const deleteUser = async (req: Request, res: Response) => {
  const { userID } = req.body;

  try {
    const   user = await prisma.users.delete({
      where: { user_id : userID },
    
    });


    res.json({ message: "User Deleted"});
  } catch (error) {
    res.status(500).json({ message: "Error Deleting User", error });
  }
};